/*--------------------------------------------------------------------------*/
/*                                                                          */
/*                                                                          */
/*      ------------         Bit-Bucket Software <no-Inc>                   */
/*      \ 10001101 /         Writers and Distributors of                    */
/*       \ 011110 /          No-Cost<no-tm> Software.                       */
/*        \ 1011 /                                                          */
/*         ------            KopyRong (K) 1987.  ALL RIGHTS REVERSED.       */
/*                                                                          */
/*                                                                          */
/*         This module was originally written by Wynn Wagner III            */
/*     with significant changes made by Vince Perriello and Bob Hartman     */
/*                                                                          */
/*                                                                          */
/*           BinkleyTerm "Main" Module: Terminal Emulation and              */
/*                       File Transfer dispatching                          */
/*                                                                          */
/*                                                                          */
/*   This  software  package is being distributed WITH FULL SOURCE CODE     */
/*   with the  following  conditions:    1)  If  anything awful happens     */
/*   because  you  use    it   (or  don't  use  it),  you  accept  full     */
/*   responsibility;  2) you  don't start making tons of voice calls to     */
/*   the authors to complain or  make  suggestions  about enhancements,     */
/*   useful or otherwise;  3) you  do not reuse this code in commercial     */
/*   products without specific permission to do so  from  the  authors;     */
/*   4) If you find any problems you send  fixes  to  the  authors  for     */
/*   inclusion  in  updates;    5) You find some way  to  express  your     */
/*   appreciation  for  this  method of distribution, either by writing     */
/*   code or  application  notes,  or  just sending along a "Thank You"     */
/*   message.                                                               */
/*                                                                          */
/*   There is  copyrighted  code  in  this product.  We either wrote it     */
/*   ourselves or got  permission  to use it.  Please don't force us to     */
/*   pay a lawyer --  have some respect for our motives and don't abuse     */
/*   this "license".                                                        */
/*                                                                          */
/*                                                                          */
/*--------------------------------------------------------------------------*/


#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include <conio.h>
#include <string.h>

#include "com.h"
#include "xfer.h"
#include "keybd.h"

struct pointers ctl;
unsigned       *carrier_mask_word = &ctl.carrier_mask;

struct baud_str btypes[] = {
                            {"300", BAUD_300},
                            {"1200", BAUD_1200},
                            {"2400", BAUD_2400},
                            {"4800", BAUD_4800},
                            {"9600", BAUD_9600},
                            {"19200", BAUD_19200},
                            {NULL, 0}
};

int             baud, cur_baud, locate_x, locate_y, port_ptr;

int             share = 0;      /* if 1, don't uninit FOSSIL  */
int             boss_net = 0;
int             boss_node = 0;
int             un_attended = 0;
int             command_line_un;
int             overwrite = 0;
int             redo_dynam = 0;
int             fullscreen = 1;
int             BBStimeout = 2000;
int             loglevel = 10;
int             gong_allowed = 0;
int             MAXport = 2;
unsigned int    comm_bits = BITS_8;
unsigned int    parity = NO_PARITY;

extern int      remote_net, remote_node;
extern struct _newnode newnodedes;      /* desc. of node             */
extern int      caller;
extern int      noforce;
extern int      old_fossil;
extern char    *keys[];
extern char     ctrlc_ctr;

char           *native_protocols = "XYSZT";     /* Protocols we support      */
char           *extern_protocols = NULL;        /* Protocols we can call     */

char           *BBSopt = NULL;  /* BBS command options       */
char           *BBSbanner = NULL;       /* BBS banner                */
char           *BBSreader = NULL;       /* Message reader */
char           *BOSSphone = NULL;       /* Phone number for boss */
char           *BOSSpwd = NULL; /* Password for boss         */
char           *download_path = "";     /* Default download path */

char           *BINKpath = "";

char            junk[100];

main (argc, argv)
int             argc;
char           *argv[];

{
   int             k;
   int             c;
   int             done;
   long            t1;
   char           *envptr, *getenv (), *malloc ();
   FILE           *logfile = NULL;      /* log file stream descriptor */
   char            prot;
   char           *p;           /* Used for external protocol */
   int             err;
   int             mail_only = 0;       /* if 1, do mail and exit     */
   int             bzone, bnet, bnode;
   extern long     timerset ();
   extern char    *receive_file ();

   envptr = getenv ("BINKLEY"); /* get path from environment */
   if ((envptr != NULL)         /* If there was one, and     */
       && (!dexists ("Binkley.Cfg")))   /* No BINKLEY.CFG locally,   */
      {
      BINKpath = malloc (strlen (envptr) + 1);  /* make room for new */
      strcpy (BINKpath, envptr);/* use BINKLEY as our path   */
      }

   signal (SIGINT, SIG_IGN);
   tzset ();

   ctl.handshake_mask = USE_XON | USE_CTS;
   ctl.carrier_mask = 0x80;
   ctl.hold_area = ctl.filepath = ctl.net_info = NULL;
   ctl.avail_list = ctl.request_list = ctl.freq_about = NULL;
   ctl.predial = ctl.postdial = ctl.m_init = NULL;
   ctl.matrix_mask = TAKE_REQ;
   ctl.system_name = ctl.sysop = NULL;
   ctl.our_zone = 1;
   ctl.max_baud = 0;

   for (k = 0; k < MAX_EXTERN; k++)     /* Zero extern proto list    */
      {
      ctl.protocols[k] = NULL;
      }

   b_init ();

   baud = 2;
   cur_baud = atoi (btypes[baud].str);
   command_line_un = 0;

   while (--argc)
      {
      ++argv;

      if (strnicmp (argv[0], "noforce", 7) == 0)
         {
         noforce = 1;
         continue;
         }

      if (strnicmp (argv[0], "mail", 4) == 0)
         {
         ++mail_only;
         continue;
         }

      if (strnicmp (argv[0], "share", 5) == 0)
         {
         ++share;
         continue;
         }

      if (strnicmp (argv[0], "dynam", 5) == 0)
         {
         ++redo_dynam;
         continue;
         }

      if (strnicmp (argv[0], "unattended", 10) == 0)
         {
         un_attended = 1;
         command_line_un = 1;
         continue;
         }

      printf ("\r\nUnrecognized option: %s\r\n", argv[0]);
      exit (1);
      }

   read_sched ();
   parse_config ();

   if (Cominit (port_ptr) != 0x1954)
      {
      cprintf ("I'm sorry, there doesn't appear to be a FOSSIL driver loaded.\r\n");
      cprintf ("BinkleyTerm requires a FOSSIL driver. Please take care of this before\r\n");
      cprintf ("attempting to run BinkleyTerm again.\r\n");
      exit (1);
      }

   k = un_attended;
   un_attended = 0;
   set_baud (ctl.max_baud, 0);
   un_attended = k;
   MDM_ENABLE (btypes[baud].rate);
   DTR_ON ();
   XON_ENABLE ();
   done = 0;

   if (mail_only)
      {
      if (fullscreen)
         {
         opening_banner ();
         mailer_banner ();
         }
      un_attended = 1;
      boss_mail (1);
      un_attended = 0;
      done = 1;
      }
   else
   if (un_attended)
      {
      done = unattended ();
      }
   else
      {
      opening_banner ();
      }

   while (!done)
      {
      c = -1;
      if (CHAR_AVAIL ())
         {
         c = MODEM_IN ();
         WRITE_ANSI (c);
         if (logfile != NULL)
            {
            fwrite (&c, 1, 1, logfile);
            }
         }
      if (KEYPRESS () || ctrlc_ctr)
         {
         c = 0;
         if (ctrlc_ctr)
            {
            /* We got a ^C, so output it properly */
            while (KEYPRESS ())		/* Flush keyboard buffer */
	       FOSSIL_CHAR ();
            ctrlc_ctr = 0;
            while (Com_Tx_NW (3) == 0)
               ;
            continue;
            }
         k = FOSSIL_CHAR ();
         switch (k)
            {
            case ALTB:
               ++baud;
               if (btypes[baud].str == NULL)
                  baud = 0;
               MDM_ENABLE (btypes[baud].rate);
               cur_baud = atoi (btypes[baud].str);
               scr_printf ("\r\nBinkleyTerm now communicating at ");
               scr_printf (btypes[baud].str);
               scr_printf (" baud.\r\n");
               break;

            case ALTC:
               scr_printf ("\r\n7 or 8 data bits? ");
               fgets (junk, 100, stdin);
               comm_bits = (atoi (junk) == 7) ? BITS_7 : BITS_8;
               if (comm_bits == BITS_8)
                  {
                  parity = NO_PARITY;
                  }
               else
                  {
                  scr_printf ("(E)ven, (O)dd, or (N)o parity? ");
                  fgets (junk, 100, stdin);
                  strupr (junk);
                  parity = (junk[0] == 'E') ? EVEN_PARITY :
                     ((junk[0] == 'O') ? ODD_PARITY : NO_PARITY);
                  }
               MDM_ENABLE (btypes[baud].rate);
               break;

            case ALTD:
               scr_printf ("\r\nPlease enter a phone (or net/node) number: ");
               if (!get_number (junk))
                  break;
               if (strchr (junk, '/') != NULL)
                  {
                  if (!nodeproc (junk))
                     break;
                  strcpy (junk, newnodedes.PhoneNumber);
                  }
               caller = 0;
               try_2_connect (junk);    /* try to connect         */
               gong ();
               break;

            case ALTH:
               t1 = timerset (300);
               while (CARRIER)
                  if (timeup (t1))
                     {
                     scr_printf ("\r\nUnable to force carrier drop by dropping DTR!\r\n");
                     break;
                     }
                  else
                     {
                     DTR_OFF ();
                     }
               DTR_ON ();
               break;

            case ALTJ:
               scr_printf ("\r\nExiting to DOS\r\n");
               scr_printf ("Type EXIT to return to BT");
               MDM_DISABLE ();
               system ("COMMAND.COM");
               scr_printf ("\r\nWelcome back!\r\n");
               /* Re-enable ourselves */
               if (Cominit (port_ptr) != 0x1954)
                  {
                  cprintf ("\r\nDarn!  The FOSSIL seems to have gone away!  Exiting\r\n");
                  exit (1);
                  }
               DTR_ON ();
               MDM_ENABLE (btypes[baud].rate);
               break;

            case ALTL:
               if (logfile != NULL)
                  {
                  fclose (logfile);
                  logfile = NULL;
                  scr_printf ("\r\nLog file closed.\r\n");
                  break;
                  }
               /* else */
               scr_printf ("\r\nLog file name: ");
               fgets (junk, 100, stdin);
               if ((k = strlen (junk)) > 2)
                  {
                  junk[--k] = '\0';     /* no '\n' */
                  logfile = fopen (junk, "ab");
                  if (logfile == NULL)
                     scr_printf ("\r\nCould not open specified logfile.\r\n");
                  }
               break;

            case ALTM:
               scr_printf ("\r\nPlease enter a net/node number: ");
               if (!get_number (junk))
                  break;
               if ((k = sscanf (junk, "%d:%d/%d", &bzone, &bnet, &bnode)) != 3)
                  {
                  bzone = ctl.our_zone;
                  k = sscanf (junk, "%d/%d", &bnet, &bnode);
                  }
               if (k >= 2)
                  {
                  if (bnet >= 0 && bnode >= 0)
                     {
                     scr_printf ("\r\nBinkleyTerm now in MAILER mode\r\n");
                     do_mail (bzone, bnet, bnode, 1);
                     scr_printf ("\r\nBinkleyTerm now in MANUAL mode\r\n");
                     gong ();
                     break;
                     }
                  }
               cprintf ("\r\nCannot get net/node from '%s'\r\n", junk);
               break;

            case ALTP:
               if (++port_ptr == MAXport)
                  port_ptr = 0;
               MDM_DISABLE ();
               Cominit (port_ptr);
               MDM_ENABLE (btypes[baud].rate);
               DTR_ON ();
               scr_printf ("\r\nBinkleyTerm now using COM");
               WRITE_ANSI (port_ptr + '1');
               scr_printf (":\r\n");
               XON_ENABLE ();
               break;

            case ALTS:
               if (old_fossil)
                  {
                  scr_printf ("\r\nRevision 3 FOSSIL driver does not support BREAK signal\r\n");
                  }
               else
                  {
                  scr_printf ("\r\nSending BREAK signal\r\n");
                  t1 = timerset (100);
                  do_break (1);
                  while (!timeup (t1))
                     time_release ();
                  do_break (0);
                  scr_printf ("Done\r\n");
                  }
               break;

            case ALTU:
               done = unattended ();
               break;

            case ALTX:
               if (logfile != NULL)
                  fclose (logfile);
               done = 1;
               break;

            case ALTY:
               scr_printf ("\r\nBinkleyTerm now in MAILER mode\r\n");
               boss_mail (1);
               scr_printf ("\r\nBinkleyTerm now in MANUAL mode\r\n");
               break;

            case PGUP:
               XON_DISABLE ();
               scr_printf ("\r\nBinkleyTerm is ready to SEND a file...\r\n");
               scr_printf ("Which Protocol do you want BinkleyTerm to use?\r\n");
               scr_printf ("Z)modem     Y)modem     S)EAlink     T)elink     X)modem\r\n");
               if (extern_protocols != NULL)
                  {
                  scr_printf (extern_protocols);
                  scr_printf ("\r\n");
                  }
               scr_printf ("OK....Choose! ");
               fgets (junk, 100, stdin);
               prot = toupper (junk[0]);
               p = NULL;
               if ((strchr (native_protocols, prot) == NULL)
                   && ((p = strchr (extern_protocols, prot)) == NULL))
                  {
                  scr_printf ("Sorry. I don't know that protocol, returning to terminal mode.\r\n");
                  break;
                  }
               scr_printf ("File to be sent? ");
               fgets (junk, 100, stdin);
               junk[strlen (junk) - 1] = '\0';
               err = 1;         /* Xmodem, Ymodem, Telink flag */
               if (p)
                  {
                  do_extern ("Send", prot, junk);
                  err = 0;
                  }
               else
               if (prot == 'Z')
                  {
                  Send_Zmodem (junk, NULL, 0, 0, 0);
                  Send_Zmodem (NULL, NULL, 0, END_BATCH, 0);
                  err = 0;
                  }
               if (prot == 'T')
                  err = xfermdm7 (junk);
               if (err)         /* 'X','Y' or good MDM7 filename */
                  {
                  send_file (junk, prot);
                  send_file (NULL, prot);
                  }
               XON_ENABLE ();
               gong ();
               break;

            case PGDN:
               XON_DISABLE ();
               scr_printf ("\r\nBinkleyTerm is ready to RECEIVE a file...\r\n");
               scr_printf ("Which Protocol do you want BinkleyTerm to use?\r\n");
               scr_printf ("Z)modem     Y)modem     S)EAlink     T)elink     X)modem\r\n");
               if (extern_protocols != NULL)
                  {
                  scr_printf (extern_protocols);
                  scr_printf ("\r\n");
                  }
               scr_printf ("OK....Choose! ");
               fgets (junk, 100, stdin);
               prot = toupper (junk[0]);
               junk[0] = '\0';
               p = NULL;
               if ((strchr (native_protocols, prot) == NULL)
                   && ((p = strchr (extern_protocols, prot)) == NULL))
                  {
                  scr_printf ("Sorry. I don't know that protocol, returning to terminal mode.\r\n");
                  break;
                  }
               err = 1;         /* Telink batch flag */
               if ((prot == 'X') || (prot == 'Y') || p != NULL)
                  {
                  scr_printf ("File to receive? ");
                  fgets (junk, 100, stdin);
                  junk[strlen (junk) - 1] = '\0';
                  if (p)
                     {
                     do_extern ("Get", prot, junk);
                     }
                  else
                     receive_file (download_path, junk, prot);
                  err = 0;
                  }
               if (prot == 'Z')
                  {
                  get_Zmodem (download_path, NULL);
                  err = 0;
                  }

               /*
                * If not Telink, err is now zero. If Telink, it should still
                * be one. 
                */
               while (err)
                  {
                  if (prot == 'T')
                     err = recvmdm7 (junk);
                  if (err)
                     err = (receive_file (download_path, junk, prot) != NULL);
                  }
               XON_ENABLE ();
               gong ();
               break;

            case ALTF10:
               main_help ();
               break;

            case ALTF1:
            case ALTF2:
            case ALTF3:
            case ALTF4:
            case ALTF5:
            case ALTF6:
            case ALTF7:
            case ALTF8:
            case ALTF9:
               c = (k >> 8) - 0x68;
               if (keys[c] != NULL)
                  {
                  SENDCHARS (keys[c], strlen (keys[c]), 0);
                  }
               break;

            default:
               c = k & 0xff;
               if (c != 0)
                  while (Com_Tx_NW (c) == 0)
                     ;
               else
                  ansi_map (k);
            }
         }
      if (c == -1)
         time_release ();
      }

   while (KEYPRESS ())
      {
      FOSSIL_CHAR ();
      }

   if (command_line_un)
      {
      mdm_init (ctl.m_busy);
      DTR_OFF ();
      }

   gotoxy (0, 22);
   if (!share)
      MDM_DISABLE ();
   cprintf ("Thanks for using BinkleyTerm.\033[K\r\nHope to see you again real soon!\033[K\r\n");
   exit (1);
}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* ansi_map() -- map function key into an ANSI escape sequence if we have   */
/* one to correspond to the key. The "emulation" is VT100-type, we map the  */
/* arrow keys to the ANSI escape sequences for up, down, right and left,    */
/* and we map the function keys into the VT100 keypad, assuming application */
/* mode, like Procomm. That is, we take the VT100 keypad, split it in two   */
/* vertically, and use the F-keys to represent the left side, and the       */
/* "shifted" F-keys to represent the right side.                            */
/*                                                                          */
/*              The result, if any, is transmitted to the host.             */
/*                                                                          */
/*--------------------------------------------------------------------------*/

char           *ansi_seq[] = {  /* IBM key	Scan code	VT100 key	 */
   "\033OP",   /* F1            3b00      PF1          */
   "\033OQ",   /* F2            3c00      PF2          */
   "\033Ow",   /* F3            3d00      keypad '7'   */
   "\033Ox",   /* F4            3e00      keypad '8'   */
   "\033Ot",   /* F5            3f00      keypad '4'   */
   "\033Ou",   /* F6            4000      keypad '5'   */
   "\033Oq",   /* F7            4100      keypad '1'   */
   "\033Or",   /* F8            4200      keypad '2'   */
   "\033Op",   /* F9            4300      keypad '0'   */
   "\033Op",   /* F10           4400      keypad '0'   */
   "",         /*               4500                   */
   "",         /*               4600                   */
   "",         /*               4700                   */
   "\033[A",   /* Up Arrow      4800      Up Arrow     */
   "",         /*               4900                   */
   "",         /*               4a00                   */
   "\033[D",   /* Left Arrow    4b00      Left Arrow   */
   "",         /*               4c00                   */
   "\033[C",   /* Right Arrow   4d00      Right Arrow  */
   "",         /*               4e00                   */
   "",         /*               4f00                   */
   "\033[B",   /* Down Arrow    5000      Down Arrow   */
   "",         /*               5100                   */
   "",         /*               5200                   */
   "",         /*               5300                   */
   "\033OR",   /* Shift-F1      5400      PF3          */
   "\033OS",   /* Shift-F2      5500      PF4          */
   "\033Oy",   /* Shift-F3      5600      keypad '9'   */
   "\033Om",   /* Shift-F4      5700      keypad '-'   */
   "\033Ov",   /* Shift-F5      5800      keypad '6'   */
   "\033Ol",   /* Shift-F6      5900      keypad ','   */
   "\033Os",   /* Shift-F7      5a00      keypad '3'   */
   "\033OM",   /* Shift-F8      5b00      keypad 'Enter' */
   "\033On",   /* Shift-F9      5c00      keypad '.'   */
   "\033OM",   /* Shift-F10     5d00      keypad 'Enter' */
   ""
};

ansi_map (ScanVal)
unsigned        ScanVal;
{
   register        KeyCode;
   register char  *s, c;

   KeyCode = ScanVal >> 8;      /* Isolate to key scan code */
   if ((KeyCode < 0x3b) || (KeyCode > 0x5d))
      return;                   /* Not in range, give up    */
   s = ansi_seq[KeyCode - 0x3b];/* Index to our sequence    */
   SENDCHARS (s, strlen (s), 0);/* Send the mapped string   */
}

opening_banner ()
{
   scr_printf ("\033[H\033[2J");
   scr_printf (ANNOUNCE);
   if (!un_attended)
      scr_printf ("A Companion Package for communicating with the Opus-CBCS\r\n");
   else
      scr_printf ("A Freely Available FidoNet Compatible Mail Utility\r\n");

   scr_printf ("Setting COM");
   WRITE_ANSI (port_ptr + '1');
   scr_printf (": to initial speed of ");
   scr_printf (btypes[baud].str);
   scr_printf (" baud. Press Alt-F10 for HELP.\r\n");
   fossil_ver ();
   mtask_find ();
}
